@@ -22,6 +22,7 @@
 struct fsl_xcvr_soc_data {
 	const char *fw_name;
 	bool spdif_only;
+	bool use_edma;
 };
 
 struct fsl_xcvr {
@@ -538,6 +539,16 @@ static int fsl_xcvr_startup(struct snd_pcm_substream *substream,
 		return -EBUSY;
 	}
 
+	/*
+	 * EDMA controller needs period size to be a multiple of
+	 * tx/rx maxburst
+	 */
+	if (xcvr->soc_data->use_edma)
+		snd_pcm_hw_constraint_step(substream->runtime, 0,
+					   SNDRV_PCM_HW_PARAM_PERIOD_SIZE,
+					   tx ? xcvr->dma_prms_tx.maxburst :
+					   xcvr->dma_prms_rx.maxburst);
+
 	switch (xcvr->mode) {
 	case FSL_XCVR_MODE_SPDIF:
 	case FSL_XCVR_MODE_ARC:
@@ -1164,28 +1175,8 @@ static irqreturn_t irq0_isr(int irq, void *devid)
 						bitrev32(val);
 				}
 				/* clear CS control register */
-				writel_relaxed(0, reg_ctrl);
+				memset_io(reg_ctrl, 0, sizeof(val));
 			}
-		} else {
-			regmap_read(xcvr->regmap, FSL_XCVR_RX_CS_DATA_0,
-				    (u32 *)&xcvr->rx_iec958.status[0]);
-			regmap_read(xcvr->regmap, FSL_XCVR_RX_CS_DATA_1,
-				    (u32 *)&xcvr->rx_iec958.status[4]);
-			regmap_read(xcvr->regmap, FSL_XCVR_RX_CS_DATA_2,
-				    (u32 *)&xcvr->rx_iec958.status[8]);
-			regmap_read(xcvr->regmap, FSL_XCVR_RX_CS_DATA_3,
-				    (u32 *)&xcvr->rx_iec958.status[12]);
-			regmap_read(xcvr->regmap, FSL_XCVR_RX_CS_DATA_4,
-				    (u32 *)&xcvr->rx_iec958.status[16]);
-			regmap_read(xcvr->regmap, FSL_XCVR_RX_CS_DATA_5,
-				    (u32 *)&xcvr->rx_iec958.status[20]);
-			for (i = 0; i < 6; i++) {
-				val = *(u32 *)(xcvr->rx_iec958.status + i * 4);
-				*(u32 *)(xcvr->rx_iec958.status + i * 4) =
-					bitrev32(val);
-			}
-			regmap_set_bits(xcvr->regmap, FSL_XCVR_RX_DPTH_CTRL,
-					FSL_XCVR_RX_DPTH_CTRL_CSA);
 		}
 	}
 	if (isr & FSL_XCVR_IRQ_NEW_UD) {
@@ -1227,6 +1218,7 @@ static const struct fsl_xcvr_soc_data fsl_xcvr_imx8mp_data = {
 
 static const struct fsl_xcvr_soc_data fsl_xcvr_imx93_data = {
 	.spdif_only = true,
+	.use_edma = true,
 };
 
 static const struct of_device_id fsl_xcvr_dt_ids[] = {
@@ -1347,10 +1339,9 @@ static int fsl_xcvr_probe(struct platform_device *pdev)
 	return ret;
 }
 
-static int fsl_xcvr_remove(struct platform_device *pdev)
+static void fsl_xcvr_remove(struct platform_device *pdev)
 {
 	pm_runtime_disable(&pdev->dev);
-	return 0;
 }
 
 static __maybe_unused int fsl_xcvr_runtime_suspend(struct device *dev)
@@ -1486,7 +1477,7 @@ static struct platform_driver fsl_xcvr_driver = {
 		.pm = &fsl_xcvr_pm_ops,
 		.of_match_table = fsl_xcvr_dt_ids,
 	},
-	.remove = fsl_xcvr_remove,
+	.remove_new = fsl_xcvr_remove,
 };
 module_platform_driver(fsl_xcvr_driver);
 
